import math

from Quartz import (
    CFURLCreateWithFileSystemPath,
    CGBitmapContextCreate,
    CGBitmapContextCreateImage,
    CGColorSpaceCreateWithName,
    CGImageDestinationAddImage,
    CGImageDestinationCreateWithURL,
    CGImageDestinationFinalize,
    CGImageSourceCopyPropertiesAtIndex,
    CGImageSourceCreateImageAtIndex,
    CGImageSourceCreateWithURL,
    CGImageSourceGetCount,
    CIContext,
    CIFilter,
    CIImage,
    CIVector,
    NSAffineTransform,
    kCFURLPOSIXPathStyle,
    kCGColorSpaceGenericRGB,
    kCGImageAlphaPremultipliedLast,
    kCIInputAspectRatioKey,
    kCIInputBackgroundImageKey,
    kCIInputImageKey,
    kCIInputScaleKey,
    kCIOutputImageKey,
)

_REMOVABLE_DISK_PATH = (
    "/System/Library/Extensions/IOStorageFamily.kext/Contents/Resources/Removable.icns"
)


def badge_disk_icon(badge_file, output_file):
    # Load the Removable disk icon
    url = CFURLCreateWithFileSystemPath(
        None, _REMOVABLE_DISK_PATH, kCFURLPOSIXPathStyle, False
    )
    backdrop = CGImageSourceCreateWithURL(url, None)
    backdropCount = CGImageSourceGetCount(backdrop)

    # Load the badge
    url = CFURLCreateWithFileSystemPath(None, badge_file, kCFURLPOSIXPathStyle, False)
    badge = CGImageSourceCreateWithURL(url, None)
    assert badge is not None, f"Unable to process image file: {badge_file}"
    badgeCount = CGImageSourceGetCount(badge)

    # Set up a destination for our target
    url = CFURLCreateWithFileSystemPath(None, output_file, kCFURLPOSIXPathStyle, False)
    target = CGImageDestinationCreateWithURL(url, "com.apple.icns", backdropCount, None)

    # Get the RGB colorspace
    rgbColorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB)

    # Scale
    scale = 1.0

    # Perspective transform
    corners = ((0.2, 0.95), (0.8, 0.95), (0.85, 0.35), (0.15, 0.35))

    # Translation
    position = (0.5, 0.5)

    for n in range(backdropCount):
        props = CGImageSourceCopyPropertiesAtIndex(backdrop, n, None)
        width = props["PixelWidth"]
        height = props["PixelHeight"]
        dpi = props["DPIWidth"]
        depth = props["Depth"]

        # Choose the best sized badge image
        bestWidth = None
        # ?? bestHeight = None
        bestBadge = None
        bestDepth = None
        # ?? bestDPI = None
        for m in range(badgeCount):
            badgeProps = CGImageSourceCopyPropertiesAtIndex(badge, m, None)
            badgeWidth = badgeProps["PixelWidth"]
            # ?? badgeHeight = badgeProps['PixelHeight']
            badgeDPI = badgeProps["DPIWidth"]
            badgeDepth = badgeProps["Depth"]

            if bestBadge is None or (
                badgeWidth <= width
                and (
                    bestWidth > width
                    or badgeWidth > bestWidth
                    or (
                        badgeWidth == bestWidth
                        and badgeDPI == dpi
                        and badgeDepth <= depth
                        and (bestDepth is None or badgeDepth > bestDepth)
                    )
                )
            ):
                bestBadge = m
                bestWidth = badgeWidth
                # ?? bestHeight = badgeHeight
                # ?? bestDPI = badgeDPI
                bestDepth = badgeDepth

        badgeImage = CGImageSourceCreateImageAtIndex(badge, bestBadge, None)
        badgeCI = CIImage.imageWithCGImage_(badgeImage)

        backgroundImage = CGImageSourceCreateImageAtIndex(backdrop, n, None)
        backgroundCI = CIImage.imageWithCGImage_(backgroundImage)

        compositor = CIFilter.filterWithName_("CISourceOverCompositing")
        lanczos = CIFilter.filterWithName_("CILanczosScaleTransform")
        perspective = CIFilter.filterWithName_("CIPerspectiveTransform")
        transform = CIFilter.filterWithName_("CIAffineTransform")

        lanczos.setValue_forKey_(badgeCI, kCIInputImageKey)
        lanczos.setValue_forKey_(scale * float(width) / bestWidth, kCIInputScaleKey)
        lanczos.setValue_forKey_(1.0, kCIInputAspectRatioKey)

        topLeft = (width * scale * corners[0][0], width * scale * corners[0][1])
        topRight = (width * scale * corners[1][0], width * scale * corners[1][1])
        bottomRight = (width * scale * corners[2][0], width * scale * corners[2][1])
        bottomLeft = (width * scale * corners[3][0], width * scale * corners[3][1])

        out = lanczos.valueForKey_(kCIOutputImageKey)
        if width >= 16:
            perspective.setValue_forKey_(out, kCIInputImageKey)
            perspective.setValue_forKey_(
                CIVector.vectorWithX_Y_(*topLeft), "inputTopLeft"
            )
            perspective.setValue_forKey_(
                CIVector.vectorWithX_Y_(*topRight), "inputTopRight"
            )
            perspective.setValue_forKey_(
                CIVector.vectorWithX_Y_(*bottomRight), "inputBottomRight"
            )
            perspective.setValue_forKey_(
                CIVector.vectorWithX_Y_(*bottomLeft), "inputBottomLeft"
            )
            out = perspective.valueForKey_(kCIOutputImageKey)

        tfm = NSAffineTransform.transform()
        tfm.translateXBy_yBy_(
            math.floor((position[0] - 0.5 * scale) * width),
            math.floor((position[1] - 0.5 * scale) * height),
        )

        transform.setValue_forKey_(out, kCIInputImageKey)
        transform.setValue_forKey_(tfm, "inputTransform")
        out = transform.valueForKey_(kCIOutputImageKey)

        compositor.setValue_forKey_(out, kCIInputImageKey)
        compositor.setValue_forKey_(backgroundCI, kCIInputBackgroundImageKey)

        result = compositor.valueForKey_(kCIOutputImageKey)

        cgContext = CGBitmapContextCreate(
            None, width, height, 8, 0, rgbColorSpace, kCGImageAlphaPremultipliedLast
        )
        context = CIContext.contextWithCGContext_options_(cgContext, None)

        context.drawImage_inRect_fromRect_(
            result, ((0, 0), (width, height)), ((0, 0), (width, height))
        )

        image = CGBitmapContextCreateImage(cgContext)

        CGImageDestinationAddImage(target, image, props)

    CGImageDestinationFinalize(target)



#!/usr/bin/env python
#
#  ds_store: Examine .DS_Store files.
#

import argparse
import os
import os.path
import pprint
import re
import sys

from ds_store import DSStore
from ds_store.buddy import BuddyError

_not_printable_re = re.compile(rb"[\x00-\x1f\x7f-\x9f]")


def usage():
    print(main.__doc__)
    sys.exit(0)


def chunks(iterable, length):
    for i in range(0, len(iterable), length):
        yield i, iterable[i : i + length]


def pretty(value):
    if isinstance(value, dict):
        return f"{{\n {pprint.pformat(value, indent=4)[1:-1]}\n}}"
    elif isinstance(value, bytearray):
        lines = ["["]
        for offset, chunk in chunks(value, 16):
            printable_chunk = _not_printable_re.sub(b".", chunk).decode("latin-1")
            hex_line = " ".join([f"{b:02x}" for b in chunk])
            line = f"  {offset:08x}  {hex_line:<47s}  {printable_chunk}"
            lines.append(line)
        lines.append("]")
        return "\n".join(lines)
    elif isinstance(value, bytes):
        return value.decode("latin-1")
    else:
        return value


def main(argv):
    """Display the contents of the .DS_Store file at the specified path.

    If you specify just a directory, ds_store will inspect the .DS_Store
    file in that directory.  If no paths are specified, ds_store will
    display the contents of the .DS_Store file in the current directory.
    """

    parser = argparse.ArgumentParser(description=main.__doc__)
    parser.add_argument("paths", nargs="*")
    args = parser.parse_args(argv)

    if len(args.paths) == 0:
        args.paths = ["."]

    failed = False
    for path in args.paths:
        if os.path.isdir(path):
            path = os.path.join(path, ".DS_Store")

        if not os.path.exists(path) or not os.path.isfile(path):
            print(f"ds_store: {path} not found", file=sys.stderr)
            failed = True
            continue

        try:
            with DSStore.open(path, "r") as d:
                print(path)
                print("")

                max_name_len = 0
                for entry in d:
                    name_len = len(entry.filename)
                    if name_len > max_name_len:
                        max_name_len = name_len

                for entry in d:
                    print(
                        "{:<{width}} {} {}".format(
                            entry.filename,
                            entry.code.decode("latin-1"),
                            pretty(entry.value),
                            width=max_name_len,
                        )
                    )
                print("")
        except BuddyError as e:
            print(f"ds_store: {path}: {e}")
            failed = True

    if failed:
        sys.exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])



import math
import re


class Color:
    def to_rgb(self):
        raise Exception("Must implement to_rgb() in subclasses")


class RGB:
    def __init__(self, r, g, b):
        self.r = r
        self.g = g
        self.b = b

    def to_rgb(self):
        return self


class HSL(Color):
    def __init__(self, h, s, l):  # noqa: E741
        self.h = h
        self.s = s
        self.l = l  # noqa: E741

    @staticmethod
    def _hue_to_rgb(t1, t2, hue):
        if hue < 0:
            hue += 6
        elif hue >= 6:
            hue -= 6

        if hue < 1:
            return (t2 - t1) * hue + t1
        elif hue < 3:
            return t2
        elif hue < 4:
            return (t2 - t1) * (4 - hue) + t1
        else:
            return t1

    def to_rgb(self):
        hue = self.h / 60.0
        if self.l <= 0.5:  # noqa: E741
            t2 = self.l * (self.s + 1)
        else:
            t2 = self.l + self.s - (self.l * self.s)
        t1 = self.l * 2 - t2
        r = self._hue_to_rgb(t1, t2, hue + 2)
        g = self._hue_to_rgb(t1, t2, hue)
        b = self._hue_to_rgb(t1, t2, hue - 2)
        return RGB(r, g, b)


class HWB(Color):
    def __init__(self, h, w, b):
        self.h = h
        self.w = w
        self.b = b

    @staticmethod
    def _hue_to_rgb(hue):
        if hue < 0:
            hue += 6
        elif hue >= 6:
            hue -= 6

        if hue < 1:
            return hue
        elif hue < 3:
            return 1
        elif hue < 4:
            return 4 - hue
        else:
            return 0

    def to_rgb(self):
        hue = self.h / 60.0
        t1 = 1 - self.w - self.b
        r = self._hue_to_rgb(hue + 2) * t1 + self.w
        g = self._hue_to_rgb(hue) * t1 + self.w
        b = self._hue_to_rgb(hue - 2) * t1 + self.w
        return RGB(r, g, b)


class CMYK(Color):
    def __init__(self, c, m, y, k):
        self.c = c
        self.m = m
        self.y = y
        self.k = k

    def to_rgb(self):
        r = 1.0 - min(1.0, self.c + self.k)
        g = 1.0 - min(1.0, self.m + self.k)
        b = 1.0 - min(1.0, self.y + self.k)
        return RGB(r, g, b)


class Gray(Color):
    def __init__(self, g):
        self.g = g

    def to_rgb(self):
        return RGB(self.g, self.g, self.g)


_x11_colors = {
    "aliceblue": (240, 248, 255),
    "antiquewhite": (250, 235, 215),
    "aqua": (0, 255, 255),
    "aquamarine": (127, 255, 212),
    "azure": (240, 255, 255),
    "beige": (245, 245, 220),
    "bisque": (255, 228, 196),
    "black": (0, 0, 0),
    "blanchedalmond": (255, 235, 205),
    "blue": (0, 0, 255),
    "blueviolet": (138, 43, 226),
    "brown": (165, 42, 42),
    "burlywood": (222, 184, 135),
    "cadetblue": (95, 158, 160),
    "chartreuse": (127, 255, 0),
    "chocolate": (210, 105, 30),
    "coral": (255, 127, 80),
    "cornflowerblue": (100, 149, 237),
    "cornsilk": (255, 248, 220),
    "crimson": (220, 20, 60),
    "cyan": (0, 255, 255),
    "darkblue": (0, 0, 139),
    "darkcyan": (0, 139, 139),
    "darkgoldenrod": (184, 134, 11),
    "darkgray": (169, 169, 169),
    "darkgreen": (0, 100, 0),
    "darkgrey": (169, 169, 169),
    "darkkhaki": (189, 183, 107),
    "darkmagenta": (139, 0, 139),
    "darkolivegreen": (85, 107, 47),
    "darkorange": (255, 140, 0),
    "darkorchid": (153, 50, 204),
    "darkred": (139, 0, 0),
    "darksalmon": (233, 150, 122),
    "darkseagreen": (143, 188, 143),
    "darkslateblue": (72, 61, 139),
    "darkslategray": (47, 79, 79),
    "darkslategrey": (47, 79, 79),
    "darkturquoise": (0, 206, 209),
    "darkviolet": (148, 0, 211),
    "deeppink": (255, 20, 147),
    "deepskyblue": (0, 191, 255),
    "dimgray": (105, 105, 105),
    "dimgrey": (105, 105, 105),
    "dodgerblue": (30, 144, 255),
    "firebrick": (178, 34, 34),
    "floralwhite": (255, 250, 240),
    "forestgreen": (34, 139, 34),
    "fuchsia": (255, 0, 255),
    "gainsboro": (220, 220, 220),
    "ghostwhite": (248, 248, 255),
    "gold": (255, 215, 0),
    "goldenrod": (218, 165, 32),
    "gray": (128, 128, 128),
    "grey": (128, 128, 128),
    "green": (0, 128, 0),
    "greenyellow": (173, 255, 47),
    "honeydew": (240, 255, 240),
    "hotpink": (255, 105, 180),
    "indianred": (205, 92, 92),
    "indigo": (75, 0, 130),
    "ivory": (255, 255, 240),
    "khaki": (240, 230, 140),
    "lavender": (230, 230, 250),
    "lavenderblush": (255, 240, 245),
    "lawngreen": (124, 252, 0),
    "lemonchiffon": (255, 250, 205),
    "lightblue": (173, 216, 230),
    "lightcoral": (240, 128, 128),
    "lightcyan": (224, 255, 255),
    "lightgoldenrodyellow": (250, 250, 210),
    "lightgray": (211, 211, 211),
    "lightgreen": (144, 238, 144),
    "lightgrey": (211, 211, 211),
    "lightpink": (255, 182, 193),
    "lightsalmon": (255, 160, 122),
    "lightseagreen": (32, 178, 170),
    "lightskyblue": (135, 206, 250),
    "lightslategray": (119, 136, 153),
    "lightslategrey": (119, 136, 153),
    "lightsteelblue": (176, 196, 222),
    "lightyellow": (255, 255, 224),
    "lime": (0, 255, 0),
    "limegreen": (50, 205, 50),
    "linen": (250, 240, 230),
    "magenta": (255, 0, 255),
    "maroon": (128, 0, 0),
    "mediumaquamarine": (102, 205, 170),
    "mediumblue": (0, 0, 205),
    "mediumorchid": (186, 85, 211),
    "mediumpurple": (147, 112, 219),
    "mediumseagreen": (60, 179, 113),
    "mediumslateblue": (123, 104, 238),
    "mediumspringgreen": (0, 250, 154),
    "mediumturquoise": (72, 209, 204),
    "mediumvioletred": (199, 21, 133),
    "midnightblue": (25, 25, 112),
    "mintcream": (245, 255, 250),
    "mistyrose": (255, 228, 225),
    "moccasin": (255, 228, 181),
    "navajowhite": (255, 222, 173),
    "navy": (0, 0, 128),
    "oldlace": (253, 245, 230),
    "olive": (128, 128, 0),
    "olivedrab": (107, 142, 35),
    "orange": (255, 165, 0),
    "orangered": (255, 69, 0),
    "orchid": (218, 112, 214),
    "palegoldenrod": (238, 232, 170),
    "palegreen": (152, 251, 152),
    "paleturquoise": (175, 238, 238),
    "palevioletred": (219, 112, 147),
    "papayawhip": (255, 239, 213),
    "peachpuff": (255, 218, 185),
    "peru": (205, 133, 63),
    "pink": (255, 192, 203),
    "plum": (221, 160, 221),
    "powderblue": (176, 224, 230),
    "purple": (128, 0, 128),
    "red": (255, 0, 0),
    "rosybrown": (188, 143, 143),
    "royalblue": (65, 105, 225),
    "saddlebrown": (139, 69, 19),
    "salmon": (250, 128, 114),
    "sandybrown": (244, 164, 96),
    "seagreen": (46, 139, 87),
    "seashell": (255, 245, 238),
    "sienna": (160, 82, 45),
    "silver": (192, 192, 192),
    "skyblue": (135, 206, 235),
    "slateblue": (106, 90, 205),
    "slategray": (112, 128, 144),
    "slategrey": (112, 128, 144),
    "snow": (255, 250, 250),
    "springgreen": (0, 255, 127),
    "steelblue": (70, 130, 180),
    "tan": (210, 180, 140),
    "teal": (0, 128, 128),
    "thistle": (216, 191, 216),
    "tomato": (255, 99, 71),
    "turquoise": (64, 224, 208),
    "violet": (238, 130, 238),
    "wheat": (245, 222, 179),
    "white": (255, 255, 255),
    "whitesmoke": (245, 245, 245),
    "yellow": (255, 255, 0),
    "yellowgreen": (154, 205, 50),
}

_ws_re = re.compile(r"\s+")
_token_re = re.compile(r"[A-Za-z_][A-Za-z0-9_]*")
_hex_re = re.compile(r"#([0-9a-f]{3}(?:[0-9a-f]{3})?)$", re.IGNORECASE)
_number_re = re.compile(r"[0-9]*(\.[0-9]*)")


class ColorParser:
    def __init__(self, s):
        self._string = s
        self._pos = 0

    def skipws(self):
        m = _ws_re.match(self._string, self._pos)
        if m:
            self._pos = m.end(0)

    def expect(self, s, context=""):
        if (
            len(self._string) - self._pos < len(s)
            or self._string[self._pos : self._pos + len(s)] != s
        ):
            raise ValueError(f'bad color "{self._string}" - expected "{s}"{context}')
        self._pos += len(s)

    def expectEnd(self):
        if self._pos != len(self._string):
            raise ValueError(f'junk at end of color "{self._string}"')

    def getToken(self):
        m = _token_re.match(self._string, self._pos)
        if m:
            token = m.group(0)

            self._pos = m.end(0)
            return token
        return None

    def parseNumber(self, context=""):
        m = _number_re.match(self._string, self._pos)
        if m:
            self._pos = m.end(0)
            return float(m.group(0))
        raise ValueError(f'bad color "{self._string}" - expected a number{context}')

    def parseColor(self):
        self.skipws()

        token = self.getToken()
        if token:
            if token == "rgb":
                return self.parseRGB()
            elif token == "hsl":
                return self.parseHSL()
            elif token == "hwb":
                return self.parseHWB()
            elif token == "cmyk":
                return self.parseCMYK()
            elif token == "gray" or token == "grey":
                return self.parseGray()

            try:
                r, g, b = _x11_colors[token]
            except KeyError:
                raise ValueError(f'unknown color name "{token}"')

            self.expectEnd()

            return RGB(r / 255.0, g / 255.0, b / 255.0)

        m = _hex_re.match(self._string, self._pos)
        if m:
            hrgb = m.group(1)

            if len(hrgb) == 3:
                r = int("0x" + 2 * hrgb[0], 16)
                g = int("0x" + 2 * hrgb[1], 16)
                b = int("0x" + 2 * hrgb[2], 16)
            else:
                r = int("0x" + hrgb[0:2], 16)
                g = int("0x" + hrgb[2:4], 16)
                b = int("0x" + hrgb[4:6], 16)

            self._pos = m.end(0)
            self.skipws()

            self.expectEnd()

            return RGB(r / 255.0, g / 255.0, b / 255.0)

        raise ValueError(f'bad color syntax "{self._string}"')

    def parseRGB(self):
        self.expect("(", 'after "rgb"')
        self.skipws()

        r = self.parseValue()

        self.skipws()
        self.expect(",", 'in "rgb"')
        self.skipws()

        g = self.parseValue()

        self.skipws()
        self.expect(",", 'in "rgb"')
        self.skipws()

        b = self.parseValue()

        self.skipws()
        self.expect(")", 'at end of "rgb"')

        self.skipws()
        self.expectEnd()

        return RGB(r, g, b)

    def parseHSL(self):
        self.expect("(", 'after "hsl"')
        self.skipws()

        h = self.parseAngle()

        self.skipws()
        self.expect(",", 'in "hsl"')
        self.skipws()

        s = self.parseValue()

        self.skipws()
        self.expect(",", 'in "hsl"')
        self.skipws()

        l = self.parseValue()  # noqa: E741

        self.skipws()
        self.expect(")", 'at end of "hsl"')

        self.skipws()
        self.expectEnd()

        return HSL(h, s, l)

    def parseHWB(self):
        self.expect("(", 'after "hwb"')
        self.skipws()

        h = self.parseAngle()

        self.skipws()
        self.expect(",", 'in "hwb"')
        self.skipws()

        w = self.parseValue()

        self.skipws()
        self.expect(",", 'in "hwb"')
        self.skipws()

        b = self.parseValue()

        self.skipws()
        self.expect(")", 'at end of "hwb"')

        self.skipws()
        self.expectEnd()

        return HWB(h, w, b)

    def parseCMYK(self):
        self.expect("(", 'after "cmyk"')
        self.skipws()

        c = self.parseValue()

        self.skipws()
        self.expect(",", 'in "cmyk"')
        self.skipws()

        m = self.parseValue()

        self.skipws()
        self.expect(",", 'in "cmyk"')
        self.skipws()

        y = self.parseValue()

        self.skipws()
        self.expect(",", 'in "cmyk"')
        self.skipws()

        k = self.parseValue()

        self.skipws()
        self.expect(")", 'at end of "cmyk"')

        self.skipws()
        self.expectEnd()

        return CMYK(c, m, y, k)

    def parseGray(self):
        self.expect("(", 'after "gray"')
        self.skipws()

        g = self.parseValue()

        self.skipws()
        self.expect(")", 'at end of "gray')

        self.skipws()
        self.expectEnd()

        return Gray(g)

    def parseValue(self):
        n = self.parseNumber()
        self.skipws()
        if self._string[self._pos] == "%":
            n = n / 100.0
            self.pos += 1
        return n

    def parseAngle(self):
        n = self.parseNumber()
        self.skipws()
        tok = self.getToken()
        if tok == "rad":
            n = n * 180.0 / math.pi
        elif tok == "grad" or tok == "gon":
            n = n * 0.9
        elif tok != "deg":
            raise ValueError(f'bad angle unit "{tok}"')
        return n


_color_re = re.compile(
    r"\s*(#|rgb|hsl|hwb|cmyk|gray|grey|{})".format("|".join(_x11_colors.keys()))
)


def isAColor(s):
    return _color_re.match(s)


def parseColor(s):
    return ColorParser(s).parseColor()



# flake8: noqa
from .alias import *
from .bookmark import *

__version__ = "2.2.2"

__all__ = [
    "__version__",
    "ALIAS_KIND_FILE",
    "ALIAS_KIND_FOLDER",
    "ALIAS_HFS_VOLUME_SIGNATURE",
    "ALIAS_FIXED_DISK",
    "ALIAS_NETWORK_DISK",
    "ALIAS_400KB_FLOPPY_DISK",
    "ALIAS_800KB_FLOPPY_DISK",
    "ALIAS_1_44MB_FLOPPY_DISK",
    "ALIAS_EJECTABLE_DISK",
    "ALIAS_NO_CNID",
    "kBookmarkPath",
    "kBookmarkCNIDPath",
    "kBookmarkFileProperties",
    "kBookmarkFileName",
    "kBookmarkFileID",
    "kBookmarkFileCreationDate",
    "kBookmarkTOCPath",
    "kBookmarkVolumePath",
    "kBookmarkVolumeURL",
    "kBookmarkVolumeName",
    "kBookmarkVolumeUUID",
    "kBookmarkVolumeSize",
    "kBookmarkVolumeCreationDate",
    "kBookmarkVolumeProperties",
    "kBookmarkContainingFolder",
    "kBookmarkUserName",
    "kBookmarkUID",
    "kBookmarkWasFileReference",
    "kBookmarkCreationOptions",
    "kBookmarkURLLengths",
    "kBookmarkSecurityExtension",
    "AppleShareInfo",
    "VolumeInfo",
    "TargetInfo",
    "Alias",
    "Bookmark",
    "Data",
    "URL",
]



#! /usr/bin/env python3
import argparse

from .core import build_dmg


def main():
    parser = argparse.ArgumentParser(description="Construct a disk image file.")
    parser.add_argument(
        "volume_name",
        metavar="volume-name",
        help=(
            "The name to give to the volume (this will appear in the title "
            "bar when the user mounts the disk image)."
        ),
    )
    parser.add_argument(
        "filename",
        metavar="output.dmg",
        help="The filename of the disk image to create.",
    )
    parser.add_argument("-s", "--settings", help="The path of the settings file.")
    parser.add_argument(
        "-D",
        dest="defines",
        action="append",
        default=[],
        help="Define a value for the settings file (e.g. -Dfoo=bar).",
    )
    parser.add_argument(
        "--no-hidpi",
        dest="lookForHiDPI",
        action="store_false",
        default=True,
        help="Do not search for HiDPI versions of the background image (if specified)",
    )
    parser.add_argument(
        "--detach-retries",
        dest="detachRetries",
        type=int,
        default=5,
        choices=range(1, 31),
        help="Number of attempts to detach disk volume (1 sec. interval)",
    )

    args = parser.parse_args()

    defines = {}
    for d in args.defines:
        k, v = d.split("=", 1)
        k = k.strip()
        v = v.strip()
        if (v.startswith("'") and v.endswith("'")) or (
            v.startswith('"') and v.endswith('"')
        ):
            v = v[1:-1]
        defines[k] = v

    build_dmg(
        args.filename,
        args.volume_name,
        args.settings,
        defines=defines,
        lookForHiDPI=args.lookForHiDPI,
        detach_retries=args.detachRetries,
    )


if __name__ == "__main__":
    main()



import os

# based on https://github.com/argv-minus-one/dmg-license/blob/master/language-info-generator/Languages.tsv
# note that this table specifies STR# Resource ID but it seems to have no effect
language_info_map = {
    "da_DK": {"buttons": "Danish", "name": "Danish", "language_id": 9},
    "de_AT": {"buttons": "German", "name": "German (Austrian)", "language_id": 92},
    "de_CH": {"buttons": "German", "name": "German (Swiss)", "language_id": 19},
    "de_DE": {"buttons": "German", "name": "German", "language_id": 3},
    "en_AU": {"buttons": "English", "name": "English (Australian)", "language_id": 15},
    "en_GB": {"buttons": "English", "name": "English (UK)", "language_id": 2},
    "en_IE": {"buttons": "English", "name": "English (Ireland)", "language_id": 108},
    "en_SG": {
        "buttons": "English",
        "name": "English (Singaporean)",
        "language_id": 100,
    },
    "en_US": {"buttons": "English", "name": "English", "language_id": 0},
    "es_ES": {"buttons": "Spanish", "name": "Spanish", "language_id": 8},
    "fi_FI": {"buttons": "Finnish", "name": "Finnish", "language_id": 17},
    "fr_BE": {"buttons": "French", "name": "French (Belgian)", "language_id": 98},
    "fr_CA": {"buttons": "French", "name": "French (Canadian)", "language_id": 11},
    "fr_CH": {"buttons": "French", "name": "French (Swiss)", "language_id": 18},
    "fr_FR": {"buttons": "French", "name": "French", "language_id": 1},
    "it_IT": {"buttons": "Italian", "name": "Italian", "language_id": 4},
    "ja_JP": {
        "buttons": "Japanese",
        "name": "Japanese",
        "language_id": 14,
        # not sure if this is correct encoding, but seems to be working
        "encoding": "shift_jis",
        "multibyte": True,
    },
    "ko_KR": {
        "buttons": "Korean",
        "name": "Korean",
        "language_id": 51,
        "encoding": "ksx1001",
        "multibyte": True,
    },
    "nb_NO": {"buttons": "Norwegian", "name": "Norwegian", "language_id": 12},
    "nl_BE": {"buttons": "Dutch", "name": "Dutch", "language_id": 6},
    "nl_NL": {"buttons": "Dutch", "name": "Dutch", "language_id": 5},
    "pt_BR": {
        "buttons": "Portuguese",
        "name": "Portuguese (Brazilian)",
        "language_id": 71,
    },
    "pt_PT": {"buttons": "Portuguese", "name": "Portuguese", "language_id": 10},
    "ru_RU": {
        "buttons": "Russian",
        "name": "Russian",
        "language_id": 49,
        "encoding": "mac_cyrillic",
    },
    "sv_SE": {"buttons": "Swedish", "name": "Swedish", "language_id": 7},
    "zh_CN": {
        "buttons": "Simplified",
        "name": "Simplified Chinese",
        "language_id": 52,
        "encoding": "gb2312",
        "multibyte": True,
    },
    "zh_TW": {
        "buttons": "Traditional",
        "name": "Traditional Chinese",
        "language_id": 53,
        "encoding": "big5",
        "multibyte": True,
    },
}

# Buttons (these come from the SLAResources file which you can find in the SLA
#          SDK on developer.apple.com)
default_buttons = {
    "English": (
        "English",
        "Agree",
        "Disagree",
        "Print",
        "Save",
        'If you agree with the terms of this license, press "Agree" to install the software.  If you do not agree, press "Disagree".',  # noqa: E501
    ),
    "German": (
        "Deutsch",
        "Akzeptieren",
        "Ablehnen",
        "Drucken",
        "Sichern...",
        'Klicken Sie in "Akzeptieren", wenn Sie mit den Bestimmungen des Software-Lizenzvertrags einverstanden sind. Falls nicht, bitte "Ablehnen" anklicken. Sie können die Software nur installieren, wenn Sie "Akzeptieren" angeklickt haben.',  # noqa: E501
    ),
    "Spanish": (
        "Español",
        "Aceptar",
        "No aceptar",
        "Imprimir",
        "Guardar...",
        'Si está de acuerdo con los términos de esta licencia, pulse "Aceptar" para instalar el software. En el supuesto de que no esté de acuerdo con los términos de esta licencia, pulse "No aceptar."',  # noqa: E501
    ),
    "French": (
        "Français",
        "Accepter",
        "Refuser",
        "Imprimer",
        "Enregistrer...",
        'Si vous acceptez les termes de la présente licence, cliquez sur "Accepter" afin d\'installer le logiciel. Si vous n\'êtes pas d\'accord avec les termes de la licence, cliquez sur "Refuser".',  # noqa: E501
    ),
    "Italian": (
        "Italiano",
        "Accetto",
        "Rifiuto",
        "Stampa",
        "Registra...",
        'Se accetti le condizioni di questa licenza, fai clic su "Accetto" per installare il software. Altrimenti fai clic su "Rifiuto".',  # noqa: E501
    ),
    "Japanese": (
        "日本語",
        "同意します",
        "同意しません",
        "印刷する",
        "保存...",
        "本ソフトウエア使用許諾契約の条件に同意される場合には、ソフトウエアをインストールするために「同意します」を押してください。\u3000同意されない場合には、「同意しません」を押してください。",  # noqa: E501
    ),
    "Dutch": (
        "Nederlands",
        "Ja",
        "Nee",
        "Print",
        "Bewaar...",
        "Indien u akkoord gaat met de voorwaarden van deze licentie, kunt u op 'Ja' klikken om de programmatuur te installeren. Indien u niet akkoord gaat, klikt u op 'Nee'.",  # noqa: E501
    ),
    "Russian": (
        "Русский",
        "Согласен",
        "Не согласен",
        "Распечатать",
        "Сохранить",
        "Если вы согласны с условиями данной лицензии, нажмите «Согласен», чтобы установить программное обеспечение. Если вы не согласны, нажмите «Не согласен».",  # noqa: E501
    ),
    "Swedish": (
        "Svensk",
        "Godkänns",
        "Avböjs",
        "Skriv ut",
        "Spara...",
        'Om Du godkänner licensvillkoren klicka på "Godkänns" för att installera programprodukten. Om Du inte godkänner licensvillkoren, klicka på "Avböjs".',  # noqa: E501
    ),
    "Portuguese": (
        "Português",
        "Concordar",
        "Discordar",
        "Imprimir",
        "Salvar...",
        'Se está de acordo com os termos desta licença, pressione "Concordar" para instalar o software. Se não está de acordo, pressione "Discordar".',  # noqa: E501
    ),
    "Simplified Chinese": (
        "汉语",
        "同意",
        "不同意",
        "打印",
        "存储…",
        "如果您同意本许可协议的条款，请按“同意”来安装此软件。如果您不同意，请按“不同意”。",  # noqa: E501
    ),
    "Traditional Chinese": (
        "漢語",
        "同意",
        "不同意",
        "列印",
        "儲存…",
        "如果您同意本許可證裡的條款，請按“同意”以安裝軟體。如果不同意，請按“不同意”。",  # noqa: E501
    ),
    "Danish": (
        "Dansk",
        "Enig",
        "Uenig",
        "Udskriv",
        "Arkiver...",
        "Hvis du accepterer betingelserne i licensaftalen, skal du klikke på “Enig” for at installere softwaren. Klik på “Uenig” for at annullere installeringen.",  # noqa: E501
    ),
    "Finnish": (
        "Suomi",
        "Hyväksyn",
        "En hyväksy",
        "Tulosta",
        "Tallenna…",
        'Hyväksy lisenssisopimuksen ehdot osoittamalla "Hyväksy". Jos et hyväksy sopimuksen ehtoja, osoita "En hyväksy".',  # noqa: E501
    ),
    "Korean": (
        "한국어",
        "동의",
        "동의 안함",
        "프린트",
        "저장...",
        '사용 계약서의 내용에 동의하면, "동의" 단추를 눌러 소프트웨어를 설치하십시오. 동의하지 않는다면, "동의 안함" 단추를 누르십시오.',  # noqa: E501
    ),
    "Norwegian": (
        "Norsk",
        "Enig",
        "Ikke enig",
        "Skriv ut",
        "Arkiver...",
        'Hvis De er enig i bestemmelsene i denne lisensavtalen, klikker De på "Enig"-knappen for å installere programvaren. Hvis De ikke er enig, klikker De på "Ikke enig".',  # noqa: E501
    ),
}

udifrezXMLtemplate = {
    "STR#": [],
    # ?? License text would be included in a block like this:
    # ?? 'TEXT': [
    # ??     {
    # ??         'Attributes': '0x0000',
    # ??         'Data': b'license text',
    # ??         'ID': '5000',
    # ??         'Name': 'English'
    # ??     }
    # ?? ],
    "TMPL": [
        {
            "Attributes": "0x0000",
            "Data": b"\x13Default Language IDDWRD\x05CountOCNT\x04****LSTC\x0bsys lang IDDWRD\x1elocal res ID (offset from 5000DWRD\x102-byte language?DWRD\x04****LSTE",  # noqa: E501
            "ID": "128",
            "Name": "LPic",
        }
    ],
}


# Another implementation in TS:
# https://github.com/argv-minus-one/dmg-license/blob/4268f2e822944fd670c1e197596396f233d6484e/src/makeLicensePlist.ts


def build_license(license_info):
    """Add a license agreement to the specified disk image file, see
    https://developer.apple.com/forums/thread/668084."""
    # Copy the original template
    xml = dict(udifrezXMLtemplate)

    licenses = license_info.get(
        "licenses",
        {"en_US": default_buttons["English"][5].encode("utf-8")},
    )

    lpic = b""
    # The first field is the default language ID.
    lpic += (5000).to_bytes(2, "big")
    # The second field is the count of language ID to license resource mappings.
    lpic += len(licenses.items()).to_bytes(2, "big")

    for language, license_data in licenses.items():
        if language not in language_info_map:
            raise Exception(
                "Unknown language '"
                + language
                + "'. Valid languages are: "
                + ", ".join(sorted(language_info_map.keys()))
            )

        language_info = language_info_map[language]
        language_buttons = language_info["buttons"]
        language_name = language_info["name"]
        language_id = language_info["language_id"]
        # for simplicity we use the same id for the resource
        # as system language id + 5000
        resource_id = language_id + 5000
        language_encoding = language_info.get("encoding", "mac_roman")
        multibyte_encoding = language_info.get("multibyte_encoding", False)

        if os.path.isfile(license_data):
            mode = "rb" if license_data.endswith(".rtf") else "r"
            with open(license_data, mode=mode) as f:
                license_data = f.read()

        if isinstance(license_data, bytes):
            if license_data.startswith(b"{\\rtf1"):
                licenseDataFormat = "RTF "
            else:
                licenseDataFormat = "TEXT"
        else:
            licenseDataFormat = "TEXT"
            license_data = license_data.encode(language_encoding)

        if licenseDataFormat not in xml:
            xml[licenseDataFormat] = []

        xml[licenseDataFormat].append(
            {
                "Attributes": "0x0000",
                "Data": license_data,
                "ID": str(resource_id),
                "Name": f"{language_name} SLA",
            }
        )

        language_default_buttons = default_buttons.get(
            language_buttons, default_buttons["English"]
        )
        buttons = license_info.get("buttons", {}).get(
            language, language_default_buttons
        )

        assert len(buttons) == 6, "License buttons must have 6 entries."

        buttons = [b.encode(language_encoding) for b in buttons]
        buttons = [len(b).to_bytes(1, "big") + b for b in buttons]
        xml["STR#"].append(
            {
                "Attributes": "0x0000",
                # \x06 is apparently the number of buttons which is always 6
                "Data": b"\x00\x06" + b"".join(buttons),
                "ID": str(resource_id),
                "Name": language_name,
            }
        )

        # Finally, the list of resource ID mappings:
        # Mapping field 1: system language ID
        lpic += language_id.to_bytes(2, "big")
        # Mapping field 2: local resource ID minus 5000
        lpic += int(resource_id - 5000).to_bytes(2, "big")
        # Mapping field 3: 2-byte language?
        lpic += int(1 if multibyte_encoding else 0).to_bytes(2, "big")

    xml["LPic"] = [
        {
            "Attributes": "0x0000",
            "Data": lpic,
            "ID": "5000",
            "Name": "",
        }
    ]
    return xml
